﻿/* SeriesBranch.cs
 * 
 * Copyright (C) 2016 E.Togashi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
using Popolo.Numerics;

namespace Popolo.HVAC.Circuit
{

  /// <summary>直列接続の回路</summary>
  public class SeriesBranch : ICircuitBranch
  {

    private CircuitNode ndUP, ndMD, ndDN;

    private ICircuitBranch brUPStrm, brDNStrm;

    /// <summary>上流節点を設定・取得する</summary>
    public CircuitNode UpStreamNode { get; set; }

    /// <summary>下流節点を設定・取得する</summary>
    public CircuitNode DownStreamNode { get; set; }

    /// <summary>体積流量[m3/s]を設定・取得する</summary>
    public double VolumetricFlowRate { get; set; }
    
    /// <summary>インスタンスを初期化する</summary>
    public SeriesBranch(ICircuitBranch brUPStrm, ICircuitBranch brDNStrm)
    {
      ndUP = new CircuitNode();
      ndMD = new CircuitNode();
      ndDN = new CircuitNode();

      this.brUPStrm = brUPStrm;
      this.brDNStrm = brDNStrm;
      brUPStrm.UpStreamNode = ndUP;
      brUPStrm.DownStreamNode = ndMD;
      brDNStrm.UpStreamNode = ndMD;
      brDNStrm.DownStreamNode = ndDN;
    }

    /// <summary>前後差圧から体積流量[m3/s]を計算する</summary>
    /// <returns>体積流量[m3/s]</returns>
    public void UpdateFlowRateFromNodePressureDifference()
    {
      double dp = UpStreamNode.Pressure - DownStreamNode.Pressure;
      ndUP.Pressure = UpStreamNode.Pressure;
      ndDN.Pressure = DownStreamNode.Pressure;

      Roots.ErrorFunction eFnc = delegate (double p)
      {
        ndMD.Pressure = p;
        brUPStrm.UpdateFlowRateFromNodePressureDifference();
        brDNStrm.UpdateFlowRateFromNodePressureDifference();
        return brUPStrm.VolumetricFlowRate - brDNStrm.VolumetricFlowRate;
      };
      Roots.Bisection(eFnc, UpStreamNode.Pressure, DownStreamNode.Pressure, 1e-7, 1e-7, 20);
      //Roots.Newton(eFnc, 0.5 * (UpStreamNode.Pressure - DownStreamNode.Pressure), 1e-6, 1e-6, 1e-5, 20);
      VolumetricFlowRate = brUPStrm.VolumetricFlowRate;
    }

  }
}
