﻿/* IAirConditioningSystemModel.cs
 * 
 * Copyright (C) 2016 E.Togashi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;

using Popolo.ThermalLoad;
using Popolo.ThermophysicalProperty;

namespace Popolo.HVAC.SubSystem
{
  /// <summary>二次側空調システム</summary>
  public interface IAirConditioningSystemModel: ImmutableAirConditioningSystemModel
  {
    /// <summary>現在の日時を設定・取得する</summary>
    new DateTime CurrentDateTime { get; set; }

    /// <summary>タイムステップを設定・取得する</summary>
    new double TimeStep { get; set; }

    /// <summary>外気条件を設定・取得する</summary>
    new ImmutableMoistAir OutdoorAir { get; set; }

    /// <summary>状態を確定する</summary>
    void FixState();
  }

  /// <summary>読み取り専用の二次側空調システム</summary>
  public interface ImmutableAirConditioningSystemModel
  {

    /// <summary>現在の日時を取得する</summary>
    DateTime CurrentDateTime { get; }

    /// <summary>タイムステップを取得する</summary>
    double TimeStep { get; }

    /// <summary>外気条件を設定・取得する</summary>
    ImmutableMoistAir OutdoorAir { get; }

    /// <summary>計算対象の建物熱負荷計算モデルを取得する</summary>
    ImmutableBuildingThermalModel BuildingThermalModel { get; }

    /// <summary>冷水往温度[C]を取得する</summary>
    double ChilledWaterSupplyTemperature { get; }

    /// <summary>冷水還温度[C]を取得する</summary>
    double ChilledWaterReturnTemperature { get; }

    /// <summary>冷水流量[kg/s]を取得する</summary>
    double ChilledWaterFlowRate { get; }

    /// <summary>温水往温度[C]を取得する</summary>
    double HotWaterSupplyTemperature { get; }

    /// <summary>温水還温度[C]を取得する</summary>
    double HotWaterReturnTemperature { get; }

    /// <summary>温水流量[kg/s]を取得する</summary>
    double HotWaterFlowRate { get; }

    /// <summary>冷温水還温度を予想する</summary>
    /// <param name="chilledWaterSupplyTemperature">冷水往温度[C]</param>
    /// <param name="hotWaterSupplyTemperature">温水往温度[C]</param>
    void ForecastReturnWaterTemperature(double chilledWaterSupplyTemperature, double hotWaterSupplyTemperature);

  }

}
