﻿using System;
using System.Collections.Generic;
using System.Text;

using Popolo.Numerics;

namespace Popolo.MachineLearning
{

  /// <summary>ニューラルネットワーク</summary>
  public class NeuralNetwork
  {
    
    //private int[] nNumbers;

    private neuron[][] neurons;

    /// <summary>コンストラクタ</summary>
    /// <param name="inputs">入力層素子数</param>
    /// <param name="outputs">出力層素子数</param>
    /// <param name="hiddenNeurons">中間層素子数リスト</param>
    public NeuralNetwork(int inputs, int outputs, int[] hiddenNeurons)
    {
      int hNum = hiddenNeurons.Length;
      neurons = new neuron[2 + hNum][];
      neurons[0] = new neuron[inputs];
      for (int i = 0; i < hiddenNeurons.Length; i++)
      {
        neurons[i + 1] = new neuron[hiddenNeurons[i]];
        for (int j = 0; j < hiddenNeurons[i]; j++)
          neurons[i + 1][j].prevNeurons = neurons[i];
      }
      neurons[hNum + 1] = new neuron[outputs];
      for (int i = 0; i < outputs; i++)
        neurons[hNum + 1][i].prevNeurons = neurons[hNum];

    }

    /// <summary>フォワード・プロパゲーション</summary>
    /// <param name="iSignals">入力信号リスト</param>
    public void ForwardPropagate(double[] iSignals)
    {
      
    }

    private class layer
    {
      public layer(layer parentLayer, layer childLayer, int numberOfNeurons)
      {
        isTopLayer = (parentLayer == null);
        isBottomLayer = (childLayer == null);

        Signals = new double[numberOfNeurons];
        ThresHolds = new double[numberOfNeurons];
        if (!isTopLayer) Weights = new double[parentLayer.Neurons, numberOfNeurons];
        else Weights = new double[0, 0];
      }
      
      private bool isTopLayer, isBottomLayer;

      public int Neurons { get { return Signals.Length; } }

      public double[,] Weights { get; }

      public double[] Signals { get; }

      public double[] ThresHolds { get; }

      public layer ParentLayer { get; private set; }

      public layer ChildLayer { get; private set; }

      public void FeedForward()
      {
        //最上位の層の場合には入力をそのまま出力する
        if (isTopLayer) return;

        //結合荷重で信号を伝達してシグモイド関数で変形する
        for (int i = 0; i < Signals.Length; i++)
        {
          Signals[i] = -ThresHolds[i];
          for (int j = 0; j < ParentLayer.Neurons; j++) Signals[i] += Weights[j, i] * ParentLayer.Signals[j];
          Signals[i] = 1 / (1 + Math.Exp(-Signals[i]));
        }
      }


      public void initWeights(NormalRandom nRnd)
      {
        //結合荷重初期化
        for (int i = 0; i < Weights.GetLength(0); i++)
          for (int j = 0; j < Weights.GetLength(1); j++) Weights[i, j] = nRnd.NextDouble();
        //閾値初期化
        for (int i = 0; i < ThresHolds.Length; i++) ThresHolds[i] = nRnd.NextDouble();
      }
    }

    private class neuron
    {
      public double threshold { get; set; }

      public double signal { get; set; }

      public neuron[] prevNeurons { get; set; }

      public double[] weights { get; set; }

      public void makeSignal()
      {
        signal = -threshold;
        for (int i = 0; i < prevNeurons.Length; i++)
          signal += prevNeurons[i].signal * weights[i];
      } 
    }

  }
}
