﻿/* SimpleHeatGain.cs
 * 
 * Copyright (C) 2016 E.Togashi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;

namespace Popolo.ThermalLoad
{
  /// <summary>発熱要素</summary>
  [Serializable]
  public class SimpleHeatGain : IHeatGain
  {
    /// <summary>顕熱対流熱取得[W]を設定・取得する</summary>
    public double ConvectiveHeatGain { get; set; }

    /// <summary>顕熱放射熱取得[W]を設定・取得する</summary>
    public double RadiativeHeatGain { get; set; }

    /// <summary>発生水分[kg/s]を設定・取得する</summary>
    public double WaterGain { get; set; }

    /// <summary>顕熱取得[W]の内、対流成分を取得する</summary>
    /// <param name="zone">発熱要素が属するゾーン</param>
    /// <returns>顕熱取得の対流成分[kW]</returns>
    public double GetConvectiveHeatGain(ImmutableZone zone) { return ConvectiveHeatGain; }

    /// <summary>顕熱取得[W]の内、放射成分を取得する</summary>
    /// <param name="zone">発熱要素が属するゾーン</param>
    /// <returns>顕熱取得の放射成分[kW]</returns>
    public double GetRadiativeHeatGain(ImmutableZone zone) { return RadiativeHeatGain; }

    /// <summary>発生水分[kg/s]を取得する</summary>
    /// <param name="zone">発熱要素が属するゾーン</param>
    /// <returns>発生水分[kg/s]</returns>
    public double GetWaterGain(ImmutableZone zone) { return WaterGain; }

    /// <summary>インスタンスを初期化する</summary>
    /// <param name="convectiveHeatGain">顕熱対流熱取得[W]</param>
    /// <param name="radiativeHeatGain">顕熱放射熱取得[W]</param>
    /// <param name="waterGain">発生水分[kg/s]</param>
    public SimpleHeatGain(double convectiveHeatGain, double radiativeHeatGain, double waterGain)
    {
      ConvectiveHeatGain = convectiveHeatGain;
      RadiativeHeatGain = radiativeHeatGain;
      WaterGain = waterGain;
    }
  }
}
